%%%%%%%%%%%%%%%%%%%%%%%%%%% STRENGTH DOMAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all
clear
clc
addpath(pwd)

%% Input

% CDP material parameters
alfa = 0.1212;
K = 2/3;
sigma_c = 6.8E6; % compressive strength                                           
sigma_t = 1.5E6; % tensile strength

% Interface parameters
mi = 0.5; % friction coefficient
c_tau = 0.5E6; % tangential cohesion
c_n = 0.5E6; % normal cohesion

% Texture parameters
beta_cell = 0.5;   % percentage of bulk material for mixed interface
f = 0.5; % 
h_b = 1; % block height
l_b = 2; % block length

% Plotting
titleFsize = 20;
titleLegFsize = 14;
labelFsize = 20;
legendFsize = 14;
tickFsize = 11;
domain_limit = [-10 6]*1.0E6;

%% GRID PARAMETERS

% Domain grid
sx = linspace(-9,2,101)*1.0E6;
sy = linspace(-9,2,101)*1.0E6;
txy = linspace(-3.5,3.5,101)*1.0E6;

% Creating meshgrid 
[SX,SY,TXY] = meshgrid(sx,sy,txy);

%% Functions

% CDP
gamma = (3*(1-K))/(2*K-1);
beta = ((sigma_c/sigma_t)*(1-alfa) - (1+alfa));
s1_eig = @(s) (s(1) + s(2))/2 + sqrt(((s(1)-s(2))/2)^2+s(3)^2);
s2_eig = @(s) (s(1) + s(2))/2 - sqrt(((s(1)-s(2))/2)^2+s(3)^2);
s3_eig = @(s) 0;
s_vec = @(s) [s1_eig(s); s2_eig(s); s3_eig(s);];
smax = @(s) max(max(s_vec(s)), 0);
p = @(s) -1/3*trace(diag(s));
S = @(s) diag(s_vec(s)) + p(s)*eye(3);
q = @(s) sqrt(3/2*sum(sum(S(s).*S(s))));
Phi_CDP = @(s) ((1/(1-alfa))*(q(s) - 3*alfa*p(s) + beta*max(smax(s),0) - gamma*max(-smax(s),0))-sigma_c);

% Horizontal
Phi_h_h = @(s) abs(s(3)) + s(2)*mi - c_tau;
Phi_h_v = @(s) s(2) - c_n;

% Diagonal
Phi_d_h = @(s) (abs(s(3)) + (s(2) + (2*h_b/l_b)*(abs(s(3))))*mi  - c_tau + (s(1) - c_n)*2*h_b/l_b);
Phi_d_v = @(s) (abs(s(3)) + (s(1) + (l_b/(2*h_b))*(abs(s(3))))*mi - c_tau + (l_b/(2*h_b))*(s(2) - c_n));

% Vertical
Phi_v = @(s) s(1) - c_n + (s(2)*mi - c_tau)*f*l_b/(2*h_b);

% Mixed
% sx_star = @(s) (s(1)<0)*s(1) + (s(1)<c*(1-beta_cell))*0 ;
sx_star = @(s) (s(1)<c_n*(1-beta_cell))*s(1) + (s(1)>=c_n*(1-beta_cell))*max(((1/beta_cell)*(s(1) - c_n*(1-beta_cell))),s(1));
tau_star = @(s) max([(abs(s(3))+s(1)*mi*(1-beta_cell) - c_tau*(1-beta_cell))/beta_cell,s(3)]);
s_star = @(s) [sx_star(s); s(2); tau_star(s);];
Phi_mixed = @(s) ((1/(1-alfa))*(q(s_star(s)) - 3*alfa*p(s_star(s)) + beta*max(smax(s_star(s)),0) - gamma*max(-smax(s_star(s)),0))-sigma_c);

% Vector of Phis
Phis = @(s) [Phi_CDP(s), Phi_h_h(s), Phi_h_v(s), Phi_d_h(s), Phi_d_v(s), Phi_v(s), Phi_mixed(s)];

% LSE 
Phi_LSE = @(s,theta) log(sum(exp(theta*Phis(s))))/theta;
Phi_max = @(s) max(Phis(s));


%% Computations

CDP = zeros(size(SX));
Horizontal_h = zeros(size(SX));
Horizontal_v = zeros(size(SX));
Diagonal_h = zeros(size(SX));
Diagonal_v = zeros(size(SX));
Vertical = zeros(size(SX));
Mixed = zeros(size(SX));
LSE = zeros(size(SX));
MAX = zeros(size(SX));
SOFTMAX = zeros(size(SX));
Ids = ones(size(SX));

nI = size(SX,1);
nJ = size(SY,1);
nK = size(TXY,1);

parfor i=1:nI
    for j=1:nJ
        for k=1:nK

            stress_state = [SX(i,j,k),SY(i,j,k),TXY(i,j,k)];
            theta = 10/(max(abs(stress_state))+1e-3);

            % Evaluating functions
            CDP(i,j,k) = Phi_CDP(stress_state);
            Horizontal_h(i,j,k) = Phi_h_h(stress_state);
            Horizontal_v(i,j,k) = Phi_h_v(stress_state);
            Diagonal_h(i,j,k) = Phi_d_h(stress_state);
            Diagonal_v(i,j,k) = Phi_d_v(stress_state);
            Vertical(i,j,k) = Phi_v(stress_state);
            Mixed(i,j,k) = Phi_mixed(stress_state);

            wv = exp(theta*[CDP(i,j,k),Horizontal_h(i,j,k),Horizontal_v(i,j,k),Diagonal_h(i,j,k),Diagonal_v(i,j,k),...
                Vertical(i,j,k),Mixed(i,j,k)]);
			wv = wv/sum(wv);
            [tmp,Ids_soft] = max(wv);
			Ids(i,j,k) = Ids_soft;

            % LSE and MAX
            LSE(i,j,k) = Phi_LSE(stress_state,theta);
            MAX(i,j,k) = Phi_max(stress_state);
            SOFTMAX(i,j,k) = wv*[CDP(i,j,k),Horizontal_h(i,j,k),Horizontal_v(i,j,k),Diagonal_h(i,j,k),Diagonal_v(i,j,k),...
                Vertical(i,j,k),Mixed(i,j,k)]';

        end
    end
end

%% Figure

figure();
hold on
isosurface(SX,SY,TXY,MAX,0,Ids);
view(3); 
camlight;
lighting gouraud; 
material metal
lightangle(gca,1.371739610594657e+02,41.070644397237508)
ax = gca; 
ax.FontSize = tickFsize;
ax.TickLabelInterpreter = 'latex';
ax.FontWeight = 'bold';
ax.GridAlpha = 0.07;
ax.LineWidth = 1;
xlim(domain_limit); ylim(domain_limit); zlim(domain_limit)
xlabel('$\sigma_{x}$ [Pa]','interpreter','latex', 'fontsize', labelFsize);
ylabel('$\sigma_{y}$ [Pa]','interpreter','latex', 'fontsize', labelFsize);
zlabel('$\tau_{xy}$ [Pa]','interpreter','latex', 'fontsize', labelFsize);
axis equal
view(135,45)
set(gca,'LineWidth',1.5)
grid on

figure();
hold on
isosurface(SX,SY,TXY,LSE,0,Ids);
view(3); 
camlight;
lighting gouraud; 
material metal
lightangle(gca,1.371739610594657e+02,41.070644397237508)
ax = gca; 
ax.FontSize = tickFsize;
ax.TickLabelInterpreter = 'latex';
ax.FontWeight = 'bold';
ax.GridAlpha = 0.07;
ax.LineWidth = 1;
xlim(domain_limit); ylim(domain_limit); zlim(domain_limit)
xlabel('$\sigma_{x}$ [Pa]','interpreter','latex', 'fontsize', labelFsize);
ylabel('$\sigma_{y}$ [Pa]','interpreter','latex', 'fontsize', labelFsize);
zlabel('$\tau_{xy}$ [Pa]','interpreter','latex', 'fontsize', labelFsize);
axis equal
view(135,45)
set(gca,'LineWidth',1.5)
grid on

figure();
hold on
isosurface(SX,SY,TXY,SOFTMAX,0,Ids);
view(3); 
camlight;
lighting gouraud; 
material metal
lightangle(gca,1.371739610594657e+02,41.070644397237508)
ax = gca; 
ax.FontSize = tickFsize;
ax.TickLabelInterpreter = 'latex';
ax.FontWeight = 'bold';
ax.GridAlpha = 0.07;
ax.LineWidth = 1;
xlim(domain_limit); ylim(domain_limit); zlim(domain_limit)
xlabel('$\sigma_{x}$ [Pa]','interpreter','latex', 'fontsize', labelFsize);
ylabel('$\sigma_{y}$ [Pa]','interpreter','latex', 'fontsize', labelFsize);
zlabel('$\tau_{xy}$ [Pa]','interpreter','latex', 'fontsize', labelFsize);
axis equal
view(135,45)
set(gca,'LineWidth',1.5)
grid on